;**********************************************************
;* File Name		:lcd.inc
;* Title		:LCD Routinen 
;* Date	 		:31.01.99
;* Version 		:1.0
;* Target MCU		:AT90S8515
;* 
;* Neu 31.1.99 / Ingo Gerlach
;* 
;* nderungen		: 
;***********************************************************
; Original von Kanada Systems , 31.1.99 geholt 
; nderungen in lcd.inc I. Gerlach 31.1.99
; Testprogramm LCDTest / I. Gerlach
; PORT A = DATA
; R S = PORT C - BIT 6
; R/W = PORT D - BIT 6
; E = PORT C - Bit 7


; **** LCD def's
; tim1-tim3 fr delay's

.def	lcd_temp	= r22
.def 	lcd_tim1	= r23
.def	lcd_tim2	= r24
.def	lcd_tim3	= r25

; **** Variablen
.equ		lcd_rs		=6		; PortC 6 LCD Dispay control bits
.equ		lcd_rw		=6		; PortD 6
.equ		lcd_en		=7		; PortC 7
.equ		lcd_function	= 0b00111000	; Function Set 8 Bit / 
						; Anzahl Zeilen /
						; Schrift (beide nicht untersttzt) 
.equ		lcd_entry	= 0b00000110	; Bewegungsrichtung / Scrollen
.equ		lcd_conb	= 0b00001110	; 1DCB / D=On,C=Cursor on,B=Cursor blinkt
.equ		lcd_len		= $0F		; Lnge der Zeile 16Zeichen

 
;** Funktionen


LCD_prepare:	ldi lcd_temp,$FF	; Port A - Ausgang
		out ddra,lcd_temp	; Datenport zum LCD
		in  lcd_temp,DDRD	; LCDWrite	
		sbr lcd_temp,$40	; PortD, 6 
		out ddrd,lcd_temp	; auf Ausgang
		in  lcd_temp,DDRC	; LCD Enable	
		sbr lcd_temp,$C0	; PortC, 6,7 
		out ddrc,lcd_temp	; 19.2.99 / Fzhler
		CBI PortC,6		; Clear RS
		CBI PortD,6		; Clear RW
		ret

lcd_coff:	Ldi R21,lcd_conb	; Cursor Ausschalten
		Cbr R21,$02		; 2. Bit lschen
		rcall lcd_control
		ret


lcd_con:	Ldi R21,lcd_conb	; Cursor Einschalten
		Sbr R21,$02		; 2. Bit Setzen
		rcall lcd_control
		ret

lcd_control:	rcall lcd_prepare
		Out PortA,R21		; Ausgabe auf Port A
		Rcall LCD_EPulse	; Enable
		Rcall Delay4ms;		; Wait 4ms		
		ret



LCD_Reset:	rcall lcd_prepare	; diverse Vorbereitungen
		Rcall delay4ms;		; Power Up Zeit ,
		Rcall delay4ms;		; nach einschalten ca. 15mS
		Rcall delay4ms;	
		Rcall delay4ms;		; Wait 15ms
		Ldi R21,lcd_function	; LCD Function Byte , sh. definition oben
		Out PortA,R21		; Ausgabe auf Port A
		Rcall LCD_EPulse	; Enable
		Rcall Delay4ms;		; Wait 4ms
		Out PortA,R21			
		Rcall LCD_EPulse
		Rcall Delay100us	; Wait 100us
		Out PortA,R21
		Rcall LCD_EPulse
		Rcall Delay100us	; Wait 100us
		Ldi R21,lcd_entry	; entry mode set
		Out PortA,R21
		Rcall LCD_EPulse
		Rcall Delay4ms		; Wait 4ms
		Ldi R21,lcd_conb	; display on
		Out PortA,R21
		Rcall LCD_EPulse
		Rcall Delay4ms;		; Wait 4ms
		Ldi R21,$01		; Clear Display
		Out PortA,R21
		Rcall LCD_EPulse
		Rcall Delay4ms		; Wait 4ms
		Ret

LCD_Write:	Cbi PortC,6
		Cbi PortD,6
		Out PortA,R23		; LCD Position
		rcall LCD_EPULSE
		Rcall Delay40us
LCD_WriteQuick:		
		Sbi PortC,6
		Out PortA,R24		; ASCII Character
		rcall LCD_EPULSE
		Rcall Delay40us
		Ret	

LCD_EPulse:	Sbi PortC,7
		nop
		nop
		nop
		Cbi PortC,7
		Ret

		

; *** diverse Delay's frs Timing ***

lc_delay:	
	
		ldi  lcd_tim2,$7f
del2:	
		ldi lcd_tim3,$7f
del1:	
		dec  lcd_tim3
		brne  del1
		dec     lcd_tim2
		brne    del2
		ret



Delay40us:	ldi lcd_tim1,$30 ; alter Wer war $40
Loop3:		dec lcd_tim1	
		brne loop3		
		ret

Delay100us:	ldi lcd_tim1,$68 ; alter Wert war $88
Loop:		dec lcd_tim1		
		brne loop		
		ret

Delay4ms:	ldi lcd_temp,$1c
Loop1:		ldi lcd_tim1,$00
Loop2:		dec lcd_tim1		
		brne loop2		
		dec lcd_temp
		brne loop1
		ret


;*********************************************************************
; Delay 2 msec with a 4MHz clock
;*********************************************************************
Delay2ms:
        ldi     lcd_tim1,$0B     
D24002: 
        ldi     lcd_tim2,$FF
D24001: dec     lcd_tim2
        brne    D24001
        dec     lcd_tim1
        brne    D24002
        ret

;*********************************************************************
; Delay 1 msec with a 4MHz clock
;*********************************************************************
Delay1ms:
        ldi     lcd_tim1,$0A     
D14002: 
        ldi     lcd_tim2,$8A
D14001: dec     lcd_tim2
        brne    D14001
        dec     lcd_tim1
        brne    D14002
        ret

delay32ms:	rcall	delay4ms
		rcall	delay4ms
		rcall	delay4ms
		rcall	delay4ms
		rcall	delay4ms
		rcall	delay4ms
		rcall	delay4ms
		rcall	delay4ms
		ret

;delay1sec:	ldi	R16,$40	
;dxloop: 	rcall	delay32mS
;		dec	r16
;		brne	dxloop	
;		ret


delay05sec:	ldi	lcd_tim3,$20	
dx2loop: 	rcall	delay32mS
		dec	lcd_tim3
		brne	dx2loop	
		ret

