;**********************************************************
;* File Name		:hfsyn01.asm
;* Title		:PLL-Steuerung mit TSA6057  / AVR 90S8515
;*			 Fuer VCO 87-147 MHz, HF-Synthesizer 0.1 - 35 MHz
;*			 Abstimmung mit Dreh-Encoder.
;*
;* 			 Per default wird von LO 87,033 MHz ausgegangen
;*			 Eine andere LO Frequenz muss ueber Setup-Mode,
;*			 Drehgeber beim Einschalten drcken, eingestellt werden.
;*			
;*			 1.2a  27.12.2001 OK, check_sweep gendert.
;*			 1.2   22.12.2001 OK, hfsyn01.asm 
;* 			 1.1a/ 16.12.2001 OK, div Probs behoben
;* 			 1.1 / 14.12.2001 Anzeige und reiner PLL Betrieb OK 
;* Version 		:1.0 / 03.12.2001 Start 
;* 
;* Target MCU		:AT90S8515
;* 
;***********************************************************

.NOLIST
.include "../Include/8515def.inc"

.list
.listmac

; Main Program Register variables
;---------------------------------------------------------------------------
.def 	temp 		= r16
.def	temp_3		= r17
.def	bits1		= r24	; fr diverse Operationen
.def	bits2		= r25	; fr diverse Operationen
.def	bits3		= r28	; fr diverse Operationen

.def	ac_tmp		= r0	;temporary storage register for SREG

;***** Subroutine Register Variables mpy16u (Math.inc)
.def	mc16uH		=r16		;multiplicand highbyte
.def	mc16uL		=r17		;multiplicand low byte
.def	mp16uH		=r18		;multiplier high byte
.def	mp16uL		=r19		;multiplier low byte
.def	m16u0		=r19		;result byte 0 (LSB)
.def	m16u1		=r18		;result byte 1
.def	m16u2		=r22		;result byte 2
.def	m16u3		=r23		;result byte 3 (MSB)
.def	mcnt16u		=r24		;loop counter

;***** Subroutine Register Variables bin2BCD32 (Math.inc)
.equ	AtBCD0		=11		;address of tBCD0
.equ	AtBCD2		=15		;address of tBCD1

.def	tBCD0		=r11		;BCD value digits 1 and 0
.def	tBCD1		=r12		;BCD value digits 3 and 2
.def	tBCD2		=r13		;BCD value digit 4
.def	tBCD3		=r14
.def	tBCD4		=r15	
.def	fbinL		=r19		;Low Byte , entspricht dem Ergebnis von mpy16u, 16u0
.def	fbinH		=r18		;binary value High byte
.def	fbinHH		=r22	
.def	fbinmsb 	=r23
.def	cnt16a		=r20		;loop counter
.def	tmp16a		=r24		;temporary value


;* I2C Devices 
;* TSA6057 PLL
.equ	tsa_mode	=  32		; FM-Mode, 32
.equ	tsa_bs		=   0 		; Bandswitch , 
.equ	tsa_adresse	= 198		; Adresse TSA6057 
.equ	tsa_raster	=   1		; Raster TSA , 1 KHz
.equ	tsa_ra01	=   0 		; 00  1KHz Raster
.equ	max_step	=   9		; 1 = 1KHz , 7 = 250 KHz,8  = 500, 9 = 1MHz
.equ	band_fo		= 0xfffe	; bei fo > 131071 geht 1KHz Raster nicht mehr, 
					; legt die max. Obergrenze der PLL fest.

;* RAM Adressen
.equ	ram_bcd		= $0060 	; Datenspeicher frt BCD Werte Frequenz
.equ	ram_div		= $0100 	; Teiler, MSB LSB, Teiler MSB-Offset , Offset = 65536 
.equ	ram_lo		= $0108 	; LO ,    MSB LSB,MSB-Offset , Offset = 65536 

.equ	ram_db		= $0110		; DatenBytes

.equ	ram_setup	= $0119		; Setup Modus = 1 
.equ	ram_ddisp	= $0120		; Debug, 1=Mode, Anzeige VCO Freq. 
.equ	ram_on_off	= $0121 	; Ablage VCO On (1) /Off (0)
.equ	ram_pll_err	= $0122 	; Fehler PLL 1 = Ja, 0 = nein
.equ	ram_tc0_dly	= $0123		; Delay Timer ISR
.equ	ram_extern	= $0124		; Steuerung PC  
.equ	ram_encoder	= $0125 	; wird auf 1 in int0isr gesetzt, Nach auslesen = 0

.equ	ram_mpy		= $0130		; Multiplyer fr Raster , Anzeige !
.equ	ram_m_ptr	= $0137		; Welcher multiplikator wird verwendet , wird ber mpy_table ausgewertet
.equ	ram_m_bcd	= $0140		; Ergebnis ram_mpy * tsa_raster als BCD

;* EEPROM Adressen
.equ	eep_lo		= $10		; EEPRom Adresse LO 00 = MSB , 01= LSB 

; Ports & Pins f. PLL
;---------------------------------------------------------------------------
.equ	func_port 	= PORTD		; port f. Up/Down 
.equ	func_pins	= PIND		; port "        "    
.equ	func_ddr	= DDRD		; data direction register for port
.equ	PFup 		= 2 		; Frequenz Up
.equ	PFdwn	 	= 4		; Frequenz Down
.equ    PMode		= 3		; Mode (Drehschalter) gedrckt
.equ    Pon_off		= 5		; Synthesizer ON/Off
.equ	SW_PORT		= PORTB
.equ	SW_PORT_DDR	= DDRB
.equ	LED_pll_ok	= 0		; Port B0 , Anzeige PLL OK
.equ	SW_on_off	= 1		; Port B1 , VCO Ein/Aus
.equ	inp_port	= PORTC		; diverse Inputs , Port C
.equ	inp_ppins	= PINC		; diverse Inputs , Port C	
.equ	inp_ddr		= DDRC		; diverse Inputs , Port C
.equ	inp_PLL_ERROR	= 0		; Input PC0 , von Komperator PLL OK ? 
.equ	inp_EXTERN	= 1		; Input PC1 , Sweep Mode ? 
.equ	inp_DEBUG	= 2		; Input PC2 , Debug Mode, Anzeige VCO

; Reload / Delay f. Timer / Counter 0 , Prfung PLL
;---------------------------------------------------------------------------
.equ	TC0_RELOAD 	= 10		; berprfung ca. alle  0,3 sec
.equ	TC0_DELAY	= 5		; 5
.equ	TC0_CK		= 5		; Teiler 1024 
					; t = (255-TC0_RELOAD)*TC0_DELAY*TEILER * T_cyl
					; T_cyl bei 4 MHz 250nS

; Registers used by I2C
; define in calling program as needed (must be >= r16)
;---------------------------------------------------------------------------
.def	bitcntr 	= r20 		; bit counter
.def	i2c_del 	= r21 		; delay loop variable
.def	i2c_buffer	= r27		; async serial data byte transmit/received
.def	bus_check	= r23		; check for bus idle

; I2C Ports
;---------------------------------------------------------------------------
.equ	i2c_delay	= 50		; wait 4.75 us at 4 MHz 25
.equ	i2c_port 	= PORTD		; port supporting i2c
.equ	i2c_pins	= PIND		; port pins supporting i2c
.equ	i2c_ddr		= DDRD		; data direction register for port
.equ	sda 		= 0		; bit within port for serial data in	
.equ	scl	 	= 1		; bit within port for serial data out


;***** Interrupt vector table

reset:
	rjmp	main			; Main rotine
	rjmp	int0isr			; External Interrupt0 handle
	rjmp	int1isr			; External Interrupt1 handle
	reti				; T/C1 Capture Event Interrupt handle
	reti				; T/C1 CompareA Interrupt handle
	reti				; T/C1 CompareB Interrupt handle
	reti				; T/C1 Overflow Interrupt handle
	rjmp	timer0_isr		; T/C0 Overflow Interrupt handle
	reti				; SPI Transfer Complete Interrupt handle
	reti				; UART Rx Complete Interrupt handle
	reti				; UART Data Register Empty Interrupt handle
	reti				; UART Tx Complete Interrupt handle
	reti				; Analog Comparator Interrupt handle


;***************************************************************************
;*
;* InitRam 2.0,  DH1AAD / 03.12.2001 
;* InitRAM Initialisiert das RAM 
;*
;***************************************************************************
initRam:				; Initialisierung mit Startwerten 
	; Datenbytes f. PLL vorbelegen . Start 15MHz , Raster 1 KHz, Teiler:102033
	ldi	Bits2,tsa_adresse	; PLL-Adresse 
	sts	ram_db+0,Bits2		; SubAdr = 0, DB0,DB1,DB2,DB3
	ldi	Bits2,0			; SubAdresse 0
	sts	ram_db+1,Bits2		; 
	ldi	Bits2,35		; Teiler DB0
	sts	ram_db+2,Bits2		; 
	ldi	Bits2,29		; Teiler DB1
	sts	ram_db+3,Bits2		
	ldi	Bits2,35		; Teiler DB2
	sts	ram_db+4,Bits2		; 
	ldi	Bits2,0			; Teiler DB3, fest auf 0
	sts	ram_db+5,Bits2		; 

	; Teilerbytes, da wenn Raster n > 16 bit (65535) werden 3 Bytes verwendet
	; wobei das 3., Byte fest ist (0x01) 65536, da fr Bereich 80-130 MHz immer gesetzt. Raster 1KHz
	; Start Freq 15.000 MHz, LO+15 = 87033+15000  = 102033
	ldi	Bits2,142
	sts	ram_div+0,Bits2		; MSB
	ldi	Bits2,145
	sts	ram_div+1,Bits2		; LSB
	ldi	Bits2,1
	sts	ram_div+2,Bits2		; MSB-Offset (S16), hier wird 1 eingetragen wenn Raster 1KHz

	; Local Oszillator
	; wobei das 3. Byte fest ist (0x01) 65536, da fr Bereich 75-90 (LO) MHz immer gesetzt.
        ; Dient weiter als untere Umschaltgrenze. Hier Ablage default. 87033, 0xAC06
	; Wenn in EEProm Wert , dann der aus EEProm

	ldi	Bits2,83
	sts	ram_lo+0,Bits2		; MSB
	ldi	Bits2,249
	sts	ram_lo+1,Bits2		; LSB
	ldi	Bits2,1
	sts	ram_lo+2,Bits2		; MSB-Offset (S16), hier wird 1 eingetragen wenn Raster 1KHz

        ; Multipl. f. Raster       
	ldi	Bits1,3			; MSB
	sts	ram_mpy+0,Bits1		; Faktor f. Raster , default 1 MHz 
	ldi	Bits1,232		; LSB
	sts	ram_mpy+1,Bits1
	ldi	Bits1,8			; 1 = Schrittweite 1 KHz, 1=1 KHz, 5 = 5 KHz , LSB 
	sts	ram_m_ptr,Bits1		; Steuert ram_mpy ( ber mpy_table) , 2 = 5, ,... , 6 = 250 max 1000

        ; VCO On / Off
	ldi	Bits2,1
	sts	ram_on_off,Bits2	; VCO on
	sts	ram_encoder,Bits2	; mit 1 belegen, neue Anzeige

	ldi	Bits2,0
	sts	ram_setup,Bits2		; Setup Mode , = 1 wenn beim Start Mode Taste gedrckt wird.
	sts	ram_pll_err,Bits2	; PLL Error , 0 = nein
	sts	ram_tc0_dly,Bits2	; Timer ISR mit 0 vorbelegen
	sts	ram_ddisp,Bits2	        ; Debug auf 0, Anzeige VCO - LO
	; sts	ram_encoder,Bits2	; mit 0 belegen, Keine neue Anzeige
	sts	ram_extern,Bits2	; mit 0 belegen, externe Steuerung = 1

	ret

; InitRAM - Ende
;***********************


;***************************************************************************
;*
;* ram_str_out /  16.12.2001 
;* Liest den an Adr. r30/r31 stehenden Text aus dem Ram und gibt ihn ab 
;* Position r23/r25 auf dem Display aus. 
;*     
;***************************************************************************
ram_str_out:
	lpm				; Lade
	mov	temp,r0
	ori	temp,$00
	brbs	1,r_stop
	mov	r24,r0
	rcall	Lcd_write		; Schreibe Zeichen
	inc	r30			; Byte Counter erhhen
	inc	r25			; LCD-Pos erhhen
	mov	r23,r25			; neue Position laden
	rjmp	ram_str_out
r_stop: ret
;***** Ende ram_str_out ******

;***************************************************************************
;*
;* show_line1 /  21.12.2001 
;* Ausgabe Zeile 1, Adresse des Strings in R31/R30
;*     
;***************************************************************************
show_line1:	ldi	R23,$80		; Position Display 2. Zeile
		ldi	r25,$80		; Position Display , 2. Zeile ,sichern zum erhhen
		rcall	ram_str_out;
		ret 
;***** Ende show_line1 ******


;***************************************************************************
;*
;* show_line2 /  21.12.2001 
;* Ausgabe Zeile 2, Adresse des Strings in R31/R30
;*     
;***************************************************************************
show_line2:	ldi	R23,$C0		; Position Display 2. Zeile
		ldi	r25,$C0		; Position Display , 2. Zeile ,sichern zum erhhen
		rcall	ram_str_out;
		ret 
;***** Ende show_line2 ******

;***************************************************************************
;*
;* show_info_Lines /  17.12.2001 
;* Zeigt nach dem Einschalten Info ber Software
;*     
;***************************************************************************

Show_Info_Lines:
 	ldi	R21,$0C			; LCD Control Display ein, Cursor aus
	rcall	lcd_control
	ldi	r31,high(info1_txt*2)   ; Pointer auf Daten , High *2 , da Programm Counter in Word's zhlt
	ldi	r30,low(info1_txt*2)	; Pointer auf Daten , Low
	rcall	show_line1;		; Ausgabe 1. zeile
	ldi	r31,high(info2_txt*2)   ; Pointer auf Daten , High *2 , da Programm Counter in Word's zhlt
	ldi	r30,low(info2_txt*2)	; Pointer auf Daten , Low
	rcall	show_line2;		; Ausgabe 2. zeile
	rcall	delay05sec;
 	ldi	R21,$01			; LCD clear Display
	rcall	lcd_control		;	
	ret

;* show_info_Lines Ende

;***************************************************************************
;*
;* show_lines
;* Zeigt Zeilen auf dem Display an 
;*     
;***************************************************************************

show_lines:
 	ldi	R21,$01			; LCD clear Display
	rcall	lcd_control		;	
	ldi	R23,$8C			; Position Display
	ldi	r25,$8C			; Position Display , sichern zum erhhen
	ldi	r31,high(mhz_txt*2);	; Pointer auf Daten , High *2 , da Programm Counter in Word's zhlt
	ldi	r30,low(mhz_txt*2)	; Pointer auf Daten , Low
	rcall	ram_str_out;
	rcall 	show_fr			; Frequenz anzeigen
	
	; 2. Zeile (Status)
	lds	Bits3,ram_on_off	; VCO Off ?
	cpi	Bits3,1			; dann Meldung Zeigen
	breq	svco_on			; 0 = Aus, 1 = ein
	rcall	Show_vco_off
	rjmp	slin_e	
svco_on:ldi	R23,$C0			; Position Display
	ldi	r25,$C0			; Position Display , sichern zum erhhen
	ldi	r31,high(step_txt*2);	; Pointer auf Daten , High *2 , da Programm Counter in Word's zhlt
	ldi	r30,low(step_txt*2)	; Pointer auf Daten , Low
	rcall	ram_str_out;
	ldi	R23,$CC			; Position Display
	ldi	r25,$CC			; Position Display , sichern zum erhhen
	ldi	r31,high(mhz_txt*2);	; Pointer auf Daten , High *2 , da Programm Counter in Word's zhlt
	ldi	r30,low(mhz_txt*2)	; Pointer auf Daten , Low
	rcall	ram_str_out;
	rcall	show_step		; Raster anzeigen
slin_e: ret

;* show_lines Ende


;***************************************************************************
;*
;* vco_on_off /  16.12.2001 
;* Schaltet Synth. Ein/Aus
;*     
;***************************************************************************
vco_on_off:
	lds	Bits3,ram_on_off
	cpi	Bits3,0
	brne	vco_off			; wenn 0 dann neu setzen 
	inc	Bits3
	cbi	SW_PORT,SW_on_off	; sonst Synth.ein 
	rjmp	vco_x
vco_off:dec	Bits3
	sbi	SW_PORT,SW_on_off	; Synth. aus
	rcall	Show_vco_off
vco_x:	sts	ram_on_off,Bits3
	rcall	show_lines		; Zeilen anzeigen
	rcall	delay32ms		; Enprellen ....	
	rcall	delay32ms		;	
	rcall	delay32ms		;	
	rcall	delay32ms		;	
	ret
;******* vco_on_off Ende

;***************************************************************************
;*
;* show_vco_off /  17.12.2001 
;* Anzeige das Synt. ausgeschaltet ist 
;*     
;***************************************************************************
Show_vco_off:
	ldi	r31,high(vco_off_txt*2) ; Pointer auf Daten , High *2 , da Programm Counter in Word's zhlt
	ldi	r30,low(vco_off_txt*2)	; Pointer auf Daten , Low
	rcall	show_line2;		; Ausgabe 2. zeile
	ret
;***** Show_vco_off Ende ******

;***************************************************************************
;*
;* pll_error /  17.12.2001 
;* Anzeige falls PLL Fehler 
;* in temp wird Status bergeben    
;*
;***************************************************************************
pll_error:
	sts	ram_pll_err,temp	; Status speichern
	cpi	temp,1			; Fehler 
	breq	perr1			
	rcall	show_lines		; sonst lschen
	rjmp	perr2			; und wech....
perr1:	ldi	r31,high(pll_err_txt*2) ; Pointer auf Daten , High *2 , da Programm Counter in Word's zhlt
	ldi	r30,low(pll_err_txt*2)	; Pointer auf Daten , Low
	rcall	show_line2;		; Ausgabe 2. zeile
perr2:	ret
;******* pll_error Ende

;***************************************************************************
;*
;* check_sweep / 21.12.01
;* Prfen ob Extern , 
;* Interupt's disabled !!!
;*
;***************************************************************************
check_sweep:
	sbic	PINC,inp_EXTERN		; Wenn low, dann extern
	rjmp	chk_swp			; sonst Ende
	ldi	r31,high(swp_mode*2)	; high, also Extern
	ldi	r30,low(swp_mode*2)	
	rcall	show_line1;		; Ausgabe 1. zeile
	ldi	r31,high(ctl_extern*2)	; 
	ldi	r30,low(ctl_extern*2)	
	rcall	show_line2;		; Ausgabe 2. zeile
chk_lp:	rcall	delay32ms
	sbis	PINC,inp_EXTERN		; high , Extern Ende
	rjmp	chk_lp;			; Endlosschleife solange Extern
	ldi	temp,1			; 1 in ram_encoder
	sts	ram_encoder,temp	; Freq. in MainLoop Aktualisieren 
	rcall	show_lines		; Anzeige aktualisieren
chk_swp:ret
;******* check_sweep Ende

;***************************************************************************
;*
;* main
;*     
;***************************************************************************
main:	ldi	temp, LOW(RAMEND)	; StackPointer Setup
	out	SPL, temp		; Initialize SPL . nicht 90S1200
	ldi	temp, HIGH(RAMEND)	; nur 90S8515
	out	SPH, temp		; Initialize SPH

	ldi	temp,0b00000000		; INT1,INT0 mask, disabled
	out	GIMSK,temp
	bclr	I			; Global interrupt disable

	rcall	InitRam			; Ram initialisieren
	rcall 	lo_chk_eep		; LO aus EEP holen, wenn gesetzt , default 87,033 MHz 

	rcall	Calc_fr			; Berechne Frequenz fr Anzeige Teiler * Raster
	rcall	SetF6057		; Setze Frequenz
	rcall	SetF6057		; Setze Frequenz

	rcall	LCD_reset		; Display initialisieren
	rcall	Show_Info_Lines		; Info's ausgeben
	rcall	show_lines		; Text VCO:      MHz anzeigen     

	
        ; *** Port Setup ***
	clr 	temp			; Port als Eingang
	out 	func_port,temp		; PortD Drehgeber und Taster
	out	inp_port,temp		; PortC 

	sbi	SW_PORT_DDR,LED_pll_ok 	; PB0 als Ausgang , Check VCO 
	sbi	SW_PORT_DDR,SW_on_off	; PB1 als Ausgang , Synth. On/Off
	;sbi	LED_PORT_DDR,LED_vco_b	; PB2 als Ausgang

	cbi	SW_PORT,SW_on_off	; Eingang, Relais 0V 

	; *** div. Input Pin's Port C  ***	
	cbi	inp_ddr,inp_PLL_ERROR;	; PC0 , Eingang Komperator
	cbi	inp_ddr,inp_EXTERN;	; PC1 , Eingang externe Steuerung
	cbi	inp_ddr,inp_DEBUG	; PC2 , Eingang Jumper , low = Debug 

	; *** Debug Jumper prfen ***
	ldi	Bits2,1			; Falls Debug				
	sbis	PINC,inp_DEBUG		; high , also normal 
	sts	ram_ddisp,Bits2	        ; Debug auf 1, Anzeige VCO - LO	

	; *** Erase EEP ***
	sbis	PIND,Pon_off		; Wenn Synth On/Of gedrckt , 0 in EEProm schreiben
	rcall	lo_eep_delete		; EEProm Lschen

	; *** Setup Mode prfen ***
	sbis	PIND,PMode		; Taste gedrckt (sbis) sbic nur zum Testen
	rcall	lo_setup		; ja , Setup vorbereiten , Lo 

	rcall	get_regs		; LED und VCO-Umschaltung initialisieren

	;*** Interrupt mask setup ***
	ldi	temp,0b11000000		; INT1,INT0 mask, enable
	out	GIMSK,temp
	;ldi	temp,0b00000011		; INT0 rising edge ,INT1 low mask, 
	ldi	temp,0b00100011		; Sleep enable, IdleMode, INT0 rising edge ,INT1 low mask, 
	out	MCUCR,temp

	ldi	temp,TC0_RELOAD		; Initialize Reload Value
	out	TCNT0,temp		; Counter vorbelegen
	ldi	temp,TC0_CK		; Teiler (5) = 1024 
	out	TCCR0, temp		; Prescaler CK/1024 
	ldi	temp,2
	out	TIMSK,temp 		; T/C0 Interrupt Enable
	
	sei				; Global interrupt enable

	; *** MainLoop	
loopA:	cli				; Whrend der Anzeige Int's sperren
	rcall	check_sweep		; Extern , prfen 
	sbis	PIND,Pon_off		; On Off gedrckt ? 
	rcall	vco_on_off		; Ein /Ausschalten
	lds	temp,ram_encoder	; 1 Wert anzeigen da nderung
	cpi	temp,0			; 0 Nichts neues da
	breq	cli_en;
	ldi	temp,0			; sonst gibt#s Mll
	sts	ram_encoder,temp	; 0 Speichern alles klar
	rcall	Calc_fr			; Neu berechnen
	rcall 	show_fr			; Frequenz anzeigen		
	;rcall	show_step
	rcall	SetF6057		; Setz Frequenz
	rcall	SetF6057		; Setz Frequenz
	rcall	SetF6057		; Setz Frequenz, mehrmals, sonst gibt es evtl. Probs
cli_en:	sei				; Int's wieder zulassen
	sleep				; CPU sleep , IDLE Mode, bis INT , timerisr ca. alle 0.06 sec
	nop				; Wenn durch Int geweckt, wird ab hier ausgefhrt.
forever: rjmp 	loopA

;***************************************************************************
;*
;* I N T E R U P T  -  R O U T I N E  N
;*     
;***************************************************************************

;***************************************************************************
;*
;* "int0isr"
;* DH1AAD  /  11.12.2001 
;* Int0-Interrupt, Drehgeber wird gedreht  **
;*     
;***************************************************************************
int0isr: 
	bclr	I			; Global interrupt disable
	in	ac_tmp,SREG		; temporarily store the Status register	
  	ldi	temp,0b00000000 	; Clear general interrupt flag register
	out	GIFR,temp	

	rcall	get_regs
	 
	sbis	PIND,PFdwn		; wenn PD4 L , dann down 
	rjmp    inc_			; sonst Up
	rjmp	dec_

dec_:	rcall	sub16			; Ergebnis in r16,r17
	rjmp	work

inc_:	rcall	add16			; Ergebnis in r16,r17

work:	sts	ram_div+1,r17		; LSB
	sts	ram_div,r16		; MSB 
	ldi	temp,1			; 1 in ram_encoder geschr. und in der 
	sts	ram_encoder,temp	; mainloop ausgewertet. Anzeige ist hier zu langesam.
	out	SREG,ac_tmp		; restore Status register
	reti				; Global interrupt enable and return
; **** Ende Int0ISR **********


;***************************************************************************
;*
;* "get_regs"
;* DH1AAD /  11.12.2001 
;* Ldt Register und vergleicht obere / untere VCO Grenze
;*     
;***************************************************************************
get_regs:lds	r18,ram_mpy+0		; MSB,
	 lds	r19,ram_mpy+1		; LSB, Faktor f. Step
	 lds	mc16uL,ram_div+1	; Teiler aus Ram lesen LSB
	 lds	mc16uH,ram_div		; Teiler aus Ram lesen MSB

	 lds	r20,ram_setup		; Setup Mode ? 
	 cpi	r20,1
	 breq	gr_wr			; Ja, Keine Prfung vco_u / vco_o

	 ldi	r20,low(band_fo)	; LSB , max. Teiler
	 cp	mc16uL,r20		; berprfung obere BandGrenze
	 ldi	r20,high(band_fo)	; MSB , max. Teiler
	 cpc	mc16uH,r20		; wenn groesser oder gleich 
	 brsh	vco_fu			; auf untere Grenze umschalten

	 lds	r20,ram_lo+1		; LSB , max. Teiler
	 inc    r20;
	 cp	mc16uL,r20		; berprfung untere BandGrenze
	 lds	r20,ram_lo+0		; MSB , max. Teiler
	 cpc	mc16uH,r20		; wenn kleiner oder gleich 
	 brlo	vco_fo			; auf obere Grenze umschalten
	
	 rjmp	gr_wr

vco_fo:	 ldi	mc16uL,low(band_fo)	; Teiler aus Ram lesen LSB LO, obere Grenze
	 ldi	mc16uH,high(band_fo)	; Teiler aus Ram lesen MSB LO
	 rjmp	gr_wr			; schreibe 0 
vco_fu:  lds	mc16uL,ram_lo+1		; Teiler aus Ram lesen LSB LO, untere Grenze
	 lds	mc16uH,ram_lo+0		; Teiler aus Ram lesen MSB LO
gr_wr:	 ret

;***************************************************************************
;*
;* mpy_table
;* DH1AAD  /  11.12.2001 
;* Zuweisung Step Pointer auf Multiplyer 
;*     
;***************************************************************************
mpy_table:
        ldi 	temp,0 
	ldi	Bits2,1			; 
	cpi	Bits1,1			; ist ram_mpy_ptr = 1 ,   1 KHz
	breq	mpy_table_end
	ldi	Bits2,5
	cpi	Bits1,2			; ist ram_mpy_ptr = 2 ,   5 KHz
	breq	mpy_table_end
	ldi	Bits2,10
	cpi	Bits1,3			; ist ram_mpy_ptr = 3 ,  10 KHz
	breq	mpy_table_end
	ldi	Bits2,50
	cpi	Bits1,4			; ist ram_mpy_ptr = 4 .  50 KHz
	breq	mpy_table_end
	ldi	Bits2,100
	cpi	Bits1,5			; ist ram_mpy_ptr = 5,  100 KHz
	breq	mpy_table_end
	ldi	Bits2,250
	cpi	Bits1,6			; ist ram_mpy_ptr = 6,  250 KHz
	breq	mpy_table_end
	ldi	Bits2,244		; LSB 500
	ldi	temp,1			; MSB 500
	cpi	Bits1,7			; ist ram_mpy_ptr = 7,  500 KHz
	breq	mpy_table_end
	ldi	Bits2,232		; LSB 1000
	ldi	temp,3			; MSB 1000
mpy_table_end:
	sts	ram_mpy+0,temp		; Faktor f. Raster , speichern   MSB     
	sts	ram_mpy+1,Bits2		; Faktor f. Raster , speichern   LSB     
	ret
	

;***************************************************************************
;*
;* "int1isr"
;* DH1AAD  /  11.12.2001 
;* Int1-Interrupt Drehgeber wird gedrckt, Menu Mode
;*     
;***************************************************************************
int1isr:
	bclr	I			;Global interrupt disable
	in	ac_tmp,SREG		;temporarily store the Status register	
	ldi	temp,0b00000000 	;Clear general interrupt flag register
	out	GIFR,temp	
	rcall	SetF6057		; Setz Frequenz
	rcall	delay32ms		
	rcall	delay32ms		
	rcall	delay32ms		
	rcall	delay32ms		
	rcall	delay32ms		
	lds	Bits1,ram_m_ptr		; Step , 1 = 1 KHz, 2 = 5 KHz usw...
	inc	Bits1			; erhhe um 1
	cpi	Bits1,max_step		; Max. erreicht ?
	brlo	r_mpy			; Lese 
	ldi	Bits1,1			; sonst wieder auf 1 
r_mpy:	sts	ram_m_ptr,Bits1		; 1..max_step , speichern
	rcall	mpy_table		; Bits2 mit Multiplyer laden
	lds	r20,ram_setup		; Setup Mode ? 
	cpi	r20,0			; Nein ? 
	breq	r_sh_l			; Normale Anzeige
	rcall	lo_eep_save		; Ja, LO Speichern
r_sh_l:	rcall	show_lines		; Anzeigen
	out	SREG,ac_tmp		; restore Status register
	reti				; Global interrupt enable and return
;**** Ende Int1ISR **********

;***************************************************************************
;*
;* "timer0_isr"
;* DH1AAD  /  21.12.2001 
;* timer0_isr, 
;* berprfung Komperator, PLL OK , Bereich 3-10 V
;* Eine aktualisierung der Anzeige findet nur statt, wenn sich auch der Status ndert. 
;* Damit wird ein 'flackern' der LCD-Anzeige verhindert. 
;*     
;***************************************************************************
timer0_isr:
	bclr	I			; Global interrupt disable
	in	ac_tmp,SREG		; temporarily store the Status register	
	ldi	temp,0b00000000 	; Clear general interrupt flag register
	out	GIFR,temp	
	lds	temp,ram_tc0_dly	; Delay aus Ram lesen 
	inc	temp			
	cpi	temp,TC0_DELAY		; Erreicht ?
	breq	t0_chk			; Ja , PLL prfen
	rjmp	t0_rex			; nein , zum ende
t0_chk:	lds	temp,ram_pll_err	; Prfen ob status gesetzt ist
	cpi	temp,1			; Ist Fehler ? 
	ldi	temp,1			; Vorbelegen 1, wird in pll_error gespeichert.
	breq	t0_err			; Ja , Status noch gesetzt
	sbis	PINC,inp_PLL_ERROR	; Status nein, prfen ob PIN
	rcall	pll_error		; Fehler, Meldung zeigen
	rjmp	t0_ret			; Raus hier ....
t0_err: ldi	temp,0			; mit 0 Vorbelegen	
	sbic	PINC,inp_PLL_ERROR	; Status ja, prfen ob Fehler weiterhin besteht
	rcall	pll_error		; nein Fehler weg, Status rcksetzen und Meldung lschen
t0_ret:	ldi	temp,0			; dly Rcksetzen
t0_rex:	sts	ram_tc0_dly,temp	; Wert in Ram speichern
	ldi	temp,TC0_RELOAD		; Initialize Reload Value
	out	TCNT0,temp		; Counter vorbelegen
	out	SREG,ac_tmp		; restore Status register
	reti
;**** Ende timer0_isr **********

;***************************************************************************
;*
;* E N D E  -  M A I N 
;*     
;***************************************************************************

;***************************************************************************
;*
;* I N C L U D E S 
;*     
;***************************************************************************
.include "hfs_6057.asm"			; Include's TSA6057 PLL
.include "hfs_lo.asm"			; Include's LO-Setup
.include "hfs_show.asm"			; Include's show .....
.include "../Include/lcd.inc"		; Include's LCD
.include "../Include/i2c.inc"		; Include's I2C
.include "../Include/math.inc"		; Include's Math-Routinen


;***************************************************************************
;*
;* C O N S T A N T E N , C S E G 
;*     
;***************************************************************************
.CSEG
;.org xxxx ; An das Ende des Progs
;                                0123456789ABCDEF
	step_txt:	.db 	"Step",0x00
	mhz_txt:	.db	"MHz",0x00
	pll_err_txt:	.db	"** PLL Error  **",0x00
        vco_off_txt:    .db     "*** Synt.Off ***",0x00
	setup_txt:    	.db     "*  Setup Mode  *",0x00
        info1_txt:    	.db     "*HF-Synthesizer*",0x00
        info2_txt:    	.db     "*Ver 1.2 12/01 *",0x00
        info3_txt:    	.db     "***  DH1AAD  ***",0x00
	info4_txt:	.db	"(c) Ingo Gerlach",0x00
	swp_mode:	.db	"** Sweep Mode **",0x00
	ctl_extern:	.db	"** E X T E R N**",0x00
;**** CSEG Daten Ende *************************

;***************************************************************************
;*
;* E E P R O M , nur fr Setup LO , wird ber Setup Routine gesetzt
;* per default LO 87,033 MHz
;*     
;***************************************************************************
;.ESEG 
;.org	eep_lo
;		eep_loa:.db	83,249		;  Test 87,033 83,249
;**** EEProm Daten Ende *************************
